#!/usr/bin/env python3

import json
import requests
import tomlkit
import urllib3
urllib3.disable_warnings()

class StatusPage:
    def do_api_calls(self):
        #services
        request_url = "{}/v1/objects/services".format(self.config['icinga2_api']['baseurl'])
        headers = {
            'Accept': 'application/json',
            'X-HTTP-Method-Override': 'GET'
        }
        requestbody = {
            "attrs": [ "name", "state", "last_check_result", "host_name", "display_name" ],
            "joins": [ "host.name", "host.state", "host.last_check_result", "host.vars" ],
            "filter": self.config['filters']['services'],
        }
        r = requests.get(request_url,
            headers=headers,
            data=json.dumps(requestbody),
            auth=(self.config['icinga2_api']['username'], self.config['icinga2_api']['password']),
            verify=False)

        if (r.status_code == 200):
            self.services = r.json()
        else:
            r.raise_for_status()


    def prettify(self, text):
        for search, replace in self.config.get('prettify', {}).items():
            text = text.replace(search, replace)

        return text


    def render_services_per_host(self, host):
        services_operational = ''
        services_warning = ''
        services_critical = ''
        card_header = ''

        services_template = """
                <li class="list-group-item d-flex justify-content-between align-items-center">
                    {}
                    <span class="badge badge-{}">{}</span>
                </li>
            """
        services_hostname_template = """
                <div id="{0}" class="card-header d-flex justify-content-between align-items-center">
                    <h4><a href="#{0}">{1}</a></h4>
                    <span class="badge badge-{2}">{3}</span>
                </div>"""

        for service in sorted(self.services['results'], key=lambda x: x['attrs']['display_name']):
            if service['attrs']['host_name'] == host:
                if service['attrs']['state'] == 0:
                    services_operational = services_operational + services_template.format(self.prettify(service['attrs']['display_name']), 'success', 'OK')
                elif service['attrs']['state'] == 1:
                    services_warning = services_warning + services_template.format(self.prettify(service['attrs']['display_name']), 'warning', 'WARNING')
                else:
                    services_critical = services_critical + services_template.format(self.prettify(service['attrs']['display_name']), 'danger', 'CRITICAL')

                if service['joins']['host']['state'] == 0:
                    card_header = services_hostname_template.format(host, service['joins']['host']['vars']['pretty_name'], 'success', 'UP')
                else:
                    card_header = services_hostname_template.format(host, service['joins']['host']['vars']['pretty_name'], 'danger', 'DOWN')

        with open("services_template.html", "r") as f:
            htmlTemplate = f.read()

        htmlOutput = htmlTemplate.format(
            card_header = card_header,
            services_operational = services_operational,
            services_warning = services_warning,
            services_critical = services_critical
        )
        return htmlOutput


    def render_service_details(self):
        # generate list of hosts by scanning services for unique host_name
        host_names = set()
        for service in self.services['results']:
            host_names.add(service['attrs']['host_name'])
        # render html for each host_name
        html_output = ""
        for host in sorted(host_names):
            html_output = html_output + self.render_services_per_host(host)
        return html_output


    def render_index_html(self, service_details):
        with open("template.html", "r") as f:
            htmlTemplate = f.read()

        htmlOutput = htmlTemplate.format(
            services = service_details
        )

        with open(self.config['output']['filename'], "w") as f:
            f.write(htmlOutput)


    def __init__(self):
        self.config = {}
        self.config['icinga2_api'] = {
            'baseurl': 'https://localhost:5665',
            'username': 'root',
            'password': 'foobar'
        }
        self.config.update(tomlkit.loads(open('config.toml').read()))

if __name__ == "__main__":
    page = StatusPage()
    page.do_api_calls()
    service_details = page.render_service_details()
    page.render_index_html(service_details)
